from audio import Audio

from owg.util.euclidian import V2F
from owg.util.euclidian import V2D
from owg.util import Calc
from owg.util.opengl import TextureLib

from bogus.core.BogusCore import egl

from mini.bossmulti import MultibossSpawner

class Peekaboo:
	def __init__(self, decal, hide_placeable):
		self.decal = decal
		self.show_pos = decal.state.pos.clone()
		self.hide_pos = V2F(hide_placeable.getX(), hide_placeable.getY())
		
	def hide_decal(self):
		self.decal.setVisible(False)
		self.decal.setLocation(self.hide_pos.x(), self.hide_pos.y(), False)
		
	def show_decal(self, speed):
		self.decal.setVisible(True)
		if (self.decal.state.pos.equals(self.show_pos)):
			return False
		return self.decal.state.pos.accelerateTowards(self.show_pos, speed).equals(self.show_pos)

borgus = Peekaboo(world["CleanBorgus"], world["CleanBorgus_Start"])
borgus_scale = borgus.decal.state.scale.clone()
shadow = Peekaboo(world["AllCleanShadow"], world["AllCleanShadow_Start"])
front = Peekaboo(world["AllCleanFront"], world["AllCleanFront_Start"])

winer_color = world["Winer"].color

state = SimpleNamespace()
state.show_borgus = False
state.show_text = False
state.intensity = 0
state.counter = 0
state.winer = False
state.winer_at_max = False

def win_game():
	game.won = True

def step():
	state.counter += 1
	borgus.decal.state.scale.set(borgus_scale).add(
		Calc.fastSin(int(state.counter / 2.5)) * 6,
		int(Calc.fastCos(int(state.counter / 3.33) - 3) * 6)
		)
	
	if (state.winer and not state.winer_at_max):
		color = winer_color.get()
		new_alpha = color.getAlpha() + 0.004
		if (new_alpha >= 1.0):
			state.winer_at_max = True
			alarm(win_game).start(60)
		winer_color.set(color.deriveAlpha(new_alpha))
	
	if (state.show_borgus and borgus.show_decal(4)):
		alarm(all_clean_now_sound).start(5)
		state.show_text = True
	if (state.intensity > 0):
		state.intensity = max(0.1, state.intensity * 0.95)
		if (state.intensity > 0.11):
			front.decal.state.pos.set(front.show_pos).add(V2F.unitRandom().multiply(state.intensity * 4))
	else:
		if (state.show_text):
			shadow.show_decal(80)
			if (front.show_decal(76)):
				alarm(show_winer).start(60)
				MultibossSpawner.win.award()
				state.intensity = 1
				
def show_winer():
	state.winer = True
				
def all_clean_now_sound():
	Audio.play("all clean now in", None, 1, None, 1)

def actually_show_borgus():
	Audio.play("borgus unsplash", None, 1, None, 1)
	state.show_borgus = True
				
def show_borgus():
	alarm(actually_show_borgus).start(30)

def start():
	winer_color.set(winer_color.get().deriveAlpha(0))
	TextureLib.tex("clean borgus").setRepeating(egl, False, False)
	borgus.hide_decal()
	shadow.hide_decal()
	front.hide_decal()

	