import random

from java.lang import System

from hubol.entity.py import PythonScriptUtils
from hubol.entity import AdvancedDecal
from hubol.entity import HubolParticle

from owg.util import Calc
from owg.util.opengl import TextureLib

from bogus.core.BogusCore import egl

bath_waters = [decal for decal in world_objects(AdvancedDecal) if decal.texture.get() == "bath water"]

def animate(alarm):
	for bath_water in bath_waters:
		bath_water.setScale(bath_water.getXScale() * -1, bath_water.getYScale(), False)

	alarm.restart(20)

def bubble(alarm):
	Bubble(
		list,
		-10,
		bath_waters[0].state.pos.clone().add(random.uniform(-180, 180), random.uniform(0, 20)),
		random.uniform(0.01, 0.02),
		random.uniform(-1, -3) if random.random() > .2 else random.uniform(1, 3),
		random.uniform(0.5, 2))
		
	alarm.restart(random.randint(3, 6))

def start():
	alarm_restartable(animate).start(20)
	alarm_restartable(bubble).start(5)

class Bubble(HubolParticle):
	def __init__(self, list, depth, pos, deltaProgress, hspeed, vspeed):
		HubolParticle.__init__(self, list, depth, pos, deltaProgress)
		self.hspeed = hspeed
		self.vspeed = vspeed
		self.boost = random.uniform(-4, -8)
		
	def step(self):
		self.state.pos.add(self.hspeed, self.boost + self.vspeed * Calc.fastSin(self.state.progress * Calc.PI * 4))
		self.boost *= 0.9
		HubolParticle.step(self)
		
	def renderLerped(self, lerped):
		mvm = egl.modelviewMatrix()
		mvm.push()
		mvm.translatef(lerped.pos.x(), lerped.pos.y(), 0)
		scale = min(1, lerped.progress * 10)
		mvm.scalef(16 * scale, 16 * scale, 1)
		egl.enableTexture2D(TextureLib.tex("bubble"), int(min(lerped.progress * 10, 9)))
		egl.unitSquare.render()
		mvm.pop()